/****************************************************************************************
*
*					timer1.c
*
*
*
*
* Basierend auf dem Code von Peter Dannegger
*****************************************************************************************/



#include "timer1.h"





/*****************************************************************************************
*							Interrupt Service Routine: TIMER1_COMPA
*
*
*
*
*Aufgabe:	Der Vergleichswert von Timer1 (OCR1A) ist so gewhlt, da er alle 4msec
*			ereicht wird und auch zum Entprellen von Tasten genutzt werden kann.
*			Diese 4msec werden zustzlich durch den Software_Teiler multipliziert, soda
*			ein Sekundenimpuls ( 4msec * 250 = 1sec) entsteht.
*
*****************************************************************************************/

ISR (TIMER1_COMPA_vect)
{

	
	if (++button_read_puls == 5)
		button_read_puls = 0;
		


	#if XTAL % SOFT_PRESCALER				// Rest? wenn ja: OCR1A = normaler Vergleichswert
		OCR1A = XTAL / SOFT_PRESCALER - 1;	// Nach dem verlngerten Vergleichswert mu wieder
	#endif									// der normale Comparewert geladen werden

	
		if(--soft_prescaler == 0)			// zhle software_teiler (250) runter bei 0 ist 
											// eine sekunde vergangen
		{
			soft_prescaler = (uint8_t) SOFT_PRESCALER;	// setze software_teiler wieder auf 
														// Anfangswert
				
			if (++second_puls == 2)		// second_puls +1, wenn 2 dann Rcksetzen 
				{second_puls = 0;}

		#if XTAL % SOFT_PRESCALER		// Rest? wenn ja: 
										//OCR1A = korrigierter Vergleiswert (nur 1x pro sec)
			OCR1A = (XTAL / SOFT_PRESCALER - 1) + XTAL % SOFT_PRESCALER
		#endif
		}
	
}



/****************************************************************************************
*
* Funktionsname:		init_timer1
*
* Rckgabe:				--
*
*Parameter:				--
*
*Aufgabe:				Initialisiert Timer1: CTC-Mode, Prescaler=1, 
*						und setzt Vergleichswert OCR1A
*						Initialisierung von Sekundenpuls
*****************************************************************************************/

void timer1_init(void)
{
	TCCR1B |= (1<<WGM12) | (1<<CS10);			// CTC-Mode, Prescaler=1
	OCR1A = XTAL / SOFT_PRESCALER - 1;			// Vergleichswert setzen
	TCNT1 = 0;									// Zhlregister initialisieren
	second_puls = 0;							// Sekundenpuls initialisieren
	button_read_puls = 0;
	soft_prescaler = (uint8_t) SOFT_PRESCALER;	// Software Teiler setzen
	TIMSK |= (1<<OCIE1A);						// Interrupt fr Comparewert A ein
}
